@@ -1,19 +1,19 @@
-import datetime
 import json
 from dataclasses import dataclass
 from functools import partial
 from itertools import starmap
-from pprint import pformat
-from typing import Any, Dict, Iterator, List, TypeVar, Union
+from typing import Any, Dict, Iterator, List, Union
 
 import rdflib
-from boltons.iterutils import remap
 from documented import DocumentedError
 from pyld.jsonld import JsonLdError
 
+from iolanta.convert_dollar_signs import convert_dollar_signs
 from iolanta.models import LDContext, LDDocument
-from octadocs.octiron.context import merge
+from iolanta.reformat_blank_nodes import reformat_blank_nodes
+from iolanta.context import merge
 from octadocs.pyld_document_loaders.choice import ChoiceLoader
+from octadocs.pyld_document_loaders.merge import MergeLoader
 from octadocs.pyld_document_loaders.named import NamedLoader
 from octadocs.types import LOCAL, Context, Triple
 from pyld import jsonld
@@ -26,10 +26,7 @@ except ImportError:
     from yaml import SafeDumper  # type: ignore   # noqa
     from yaml import SafeLoader  # type: ignore   # noqa
 
-
-MetaData = Union[List[Dict[str, Any]], Dict[str, Any]]   # type: ignore  # noqa
-
-Data = TypeVar('Data')
+MetaData = Union[List[Dict[str, Any]], Dict[str, Any]]  # type: ignore  # noqa
 
 
 @dataclass
@@ -37,11 +34,11 @@ class ExpandError(DocumentedError):
     """
     JSON-LD expand operation failed.
 
-    {self.message}
-
     Data: {self.formatted_data}
     Context: {self.formatted_context}
     IRI: {self.iri}
+
+    {self.message}
     """
 
     message: str
@@ -60,41 +57,6 @@ class ExpandError(DocumentedError):
         return json.dumps(self.context, indent=2)
 
 
-def _convert(term: Any) -> Any:  # type: ignore
-    """Convert $statement to @statement."""
-    if isinstance(term, str) and term.startswith('$'):
-        return '@' + term[1:]  # noqa: WPS336
-
-    # pyld cannot expand() a document which contains data which cannot be
-    # trivially serialized to JSON.
-    # FIXME we may want to replace this with an xsd:date declaration, but let's
-    #   revisit that later.
-    if isinstance(term, (datetime.date, datetime.datetime)):
-        return str(term)
-
-    return term
-
-
-def convert_dollar_signs(
-    meta_data: Data,
-) -> Data:
-    """
-    Convert $ character to @ in keys.
-
-    We use $ by convention to avoid writing quotes.
-    """
-    if not meta_data:
-        return {}
-
-    return remap(
-        meta_data,
-        lambda path, key, value: (  # noqa: WPS110
-            _convert(key),
-            _convert(value),
-        ),
-    )
-
-
 def as_triple_stream(
     raw_data: MetaData,
     context: Context,
@@ -125,24 +87,30 @@ def as_triple_stream(
         raise ValueError(f'Format of data not recognized: {raw_data}')
 
 
-def reformat_blank_nodes(prefix: str, triple: Triple) -> Triple:
-    """
-    Prepend prefix to every blank node.
+def jsonld_expand(data, context, local_iri, pyld_loader=None):
+    """Expand the JSON_LD data with the help of particular loader."""
+    options = {
+        'base': str(LOCAL),
+    }
 
-    JSON-LD flattening creates sequential identifiers in the form of _:bN
-    for every blank node. See https://www.w3.org/TR/json-ld11/
+    if context:
+        options.update({'expandContext': context})
 
-    This means that subgraphs of the Octiron ConjunctiveGraph have clashing
-    blank node identifiers.
+    if pyld_loader is not None:
+        options.update({'documentLoader': pyld_loader})
 
-    To avoid that, we prepend a prefix to every blank node identifier.
-    """
-    return Triple(*(
-        rdflib.BNode(value=prefix + str(singleton)) if (
-            isinstance(singleton, rdflib.BNode)
-        ) else singleton
-        for singleton in triple
-    ))
+    try:
+        return jsonld.expand(
+            data,
+            options=options,
+        )
+    except (TypeError, JsonLdError) as err:
+        raise ExpandError(
+            message=str(err),
+            document=data,
+            context=context,
+            iri=local_iri,
+        ) from err
 
 
 def _dict_as_triple_stream(  # type: ignore
@@ -182,25 +150,18 @@ def _dict_as_triple_stream(  # type: ignore
 
     # Reason: https://github.com/RDFLib/rdflib-jsonld/issues/97
     # If we don't expand with an explicit @base, import will fail silently.
-    try:
-        meta_data = jsonld.expand(
-            meta_data,
-            options={
-                'base': str(LOCAL),
-                'expandContext': context,
-                'documentLoader': ChoiceLoader(loaders=[
-                     NamedLoader(documents=named_contexts),
-                     requests_document_loader(),
-                ]),
-            },
+    meta_data = jsonld_expand(
+        data=meta_data,
+        local_iri=local_iri,
+        context=context,
+        pyld_loader=MergeLoader(
+            base_context=context,
+            loader=ChoiceLoader(loaders=[
+                NamedLoader(documents=named_contexts),
+                requests_document_loader(),
+            ]),
         )
-    except (TypeError, JsonLdError) as err:
-        raise ExpandError(
-            message=str(err),
-            document=meta_data,
-            context=context,
-            iri=local_iri,
-        ) from err
+    )
 
     # Reason: https://github.com/RDFLib/rdflib-jsonld/issues/98
     # If we don't flatten, @included sections will not be imported.
